@@ -19,24 +19,23 @@ package org.apache.kyuubi.engine.spark.udf
 
 import java.nio.file.Paths
 
-import org.apache.kyuubi.{KyuubiFunSuite, MarkdownBuilder, MarkdownUtils, Utils}
+import org.apache.kyuubi.{KyuubiFunSuite, MarkdownBuilder, Utils}
+import org.apache.kyuubi.util.GoldenFileUtils._
 
-// scalastyle:off line.size.limit
 /**
  * End-to-end test cases for configuration doc file
- * The golden result file is "docs/sql/functions.md".
+ * The golden result file is "docs/extensions/engines/spark/functions.md".
  *
  * To run the entire test suite:
  * {{{
- *   build/mvn clean test -pl externals/kyuubi-spark-sql-engine -am -Pflink-provided,spark-provided,hive-provided -DwildcardSuites=org.apache.kyuubi.engine.spark.udf.KyuubiDefinedFunctionSuite
+ *   KYUUBI_UPDATE=0 dev/gen/gen_spark_kdf_docs.sh
  * }}}
  *
  * To re-generate golden files for entire suite, run:
  * {{{
- *   KYUUBI_UPDATE=1 build/mvn clean test -pl externals/kyuubi-spark-sql-engine -am -Pflink-provided,spark-provided,hive-provided -DwildcardSuites=org.apache.kyuubi.engine.spark.udf.KyuubiDefinedFunctionSuite
+ *   dev/gen/gen_spark_kdf_docs.sh
  * }}}
  */
-// scalastyle:on line.size.limit
 class KyuubiDefinedFunctionSuite extends KyuubiFunSuite {
 
   private val kyuubiHome: String = Utils.getCodeSourceLocation(getClass)
@@ -48,23 +47,18 @@ class KyuubiDefinedFunctionSuite extends KyuubiFunSuite {
   test("verify or update kyuubi spark sql functions") {
     val builder = MarkdownBuilder(licenced = true, getClass.getName)
 
-    builder
-      .line("# Auxiliary SQL Functions")
-      .line("""Kyuubi provides several auxiliary SQL functions as supplement to Spark's
+    builder += "# Auxiliary SQL Functions" +=
+      """Kyuubi provides several auxiliary SQL functions as supplement to Spark's
         | [Built-in Functions](https://spark.apache.org/docs/latest/api/sql/index.html#
-        |built-in-functions)""")
-      .lines("""
+        |built-in-functions)""" ++=
+      """
         | Name | Description | Return Type | Since
         | --- | --- | --- | ---
-        |""")
+        |"""
     KDFRegistry.registeredFunctions.foreach { func =>
-      builder.line(s"${func.name} | ${func.description} | ${func.returnType} | ${func.since}")
+      builder += s"${func.name} | ${func.description} | ${func.returnType} | ${func.since}"
     }
 
-    MarkdownUtils.verifyOutput(
-      markdown,
-      builder,
-      getClass.getCanonicalName,
-      "externals/kyuubi-spark-sql-engine")
+    verifyOrRegenerateGoldenFile(markdown, builder.toMarkdown, "dev/gen/gen_spark_kdf_docs.sh")
   }
 }
