@@ -29,9 +29,7 @@ import org.apache.kyuubi.operation.HiveJDBCTestHelper
 
 class SQLOperationListenerSuite extends WithSparkSQLEngine with HiveJDBCTestHelper {
 
-  override def withKyuubiConf: Map[String, String] = Map(
-    KyuubiConf.ENGINE_SPARK_SHOW_PROGRESS.key -> "true",
-    KyuubiConf.ENGINE_SPARK_SHOW_PROGRESS_UPDATE_INTERVAL.key -> "200")
+  override def withKyuubiConf: Map[String, String] = Map.empty
 
   override protected def jdbcUrl: String = getJdbcUrl
 
@@ -58,19 +56,23 @@ class SQLOperationListenerSuite extends WithSparkSQLEngine with HiveJDBCTestHelp
   }
 
   test("operation listener with progress job info") {
-    val sql = "SELECT java_method('java.lang.Thread', 'sleep', 10000l) FROM range(1, 3, 1, 2);"
-    withSessionHandle { (client, handle) =>
-      val req = new TExecuteStatementReq()
-      req.setSessionHandle(handle)
-      req.setStatement(sql)
-      val tExecuteStatementResp = client.ExecuteStatement(req)
-      val opHandle = tExecuteStatementResp.getOperationHandle
-      val fetchResultsReq = new TFetchResultsReq(opHandle, TFetchOrientation.FETCH_NEXT, 1000)
-      fetchResultsReq.setFetchType(1.toShort)
-      eventually(timeout(90.seconds), interval(500.milliseconds)) {
-        val resultsResp = client.FetchResults(fetchResultsReq)
-        val logs = resultsResp.getResults.getColumns.get(0).getStringVal.getValues.asScala
-        assert(logs.exists(_.matches(".*\\[Job .* Stages\\] \\[Stage .*\\]")))
+    withSessionConf(Map(
+      KyuubiConf.ENGINE_SPARK_SHOW_PROGRESS.key -> "true",
+      KyuubiConf.ENGINE_SPARK_SHOW_PROGRESS_UPDATE_INTERVAL.key -> "200"))()() {
+      val sql = "SELECT java_method('java.lang.Thread', 'sleep', 10000l) FROM range(1, 3, 1, 2);"
+      withSessionHandle { (client, handle) =>
+        val req = new TExecuteStatementReq()
+        req.setSessionHandle(handle)
+        req.setStatement(sql)
+        val tExecuteStatementResp = client.ExecuteStatement(req)
+        val opHandle = tExecuteStatementResp.getOperationHandle
+        val fetchResultsReq = new TFetchResultsReq(opHandle, TFetchOrientation.FETCH_NEXT, 1000)
+        fetchResultsReq.setFetchType(1.toShort)
+        eventually(timeout(90.seconds), interval(500.milliseconds)) {
+          val resultsResp = client.FetchResults(fetchResultsReq)
+          val logs = resultsResp.getResults.getColumns.get(0).getStringVal.getValues.asScala
+          assert(logs.exists(_.matches(".*\\[Job .* Stages\\] \\[Stage .*\\]")))
+        }
       }
     }
   }
