@@ -41,6 +41,7 @@ import org.apache.kyuubi.server.metadata.{MetadataManager, MetadataRequestsRetry
 import org.apache.kyuubi.server.metadata.api.{Metadata, MetadataFilter}
 import org.apache.kyuubi.sql.parser.server.KyuubiParser
 import org.apache.kyuubi.util.{SignUtils, ThreadUtils}
+import org.apache.kyuubi.util.ThreadUtils.scheduleTolerableRunnableWithFixedDelay
 
 class KyuubiSessionManager private (name: String) extends SessionManager(name) {
 
@@ -396,20 +397,22 @@ class KyuubiSessionManager private (name: String) extends SessionManager(name) {
   private def startEngineAliveChecker(): Unit = {
     val interval = conf.get(KyuubiConf.ENGINE_ALIVE_PROBE_INTERVAL)
     val checkTask: Runnable = () => {
-      allSessions().foreach { session =>
-        if (!session.asInstanceOf[KyuubiSessionImpl].checkEngineConnectionAlive()) {
+      allSessions().foreach {
+        case session: KyuubiSessionImpl =>
           try {
-            closeSession(session.handle)
-            logger.info(s"The session ${session.handle} has been closed " +
-              s"due to engine unresponsiveness (checked by the engine alive checker).")
+            if (!session.checkEngineConnectionAlive()) {
+              closeSession(session.handle)
+              logger.info(s"The session ${session.handle} has been closed " +
+                s"due to engine unresponsiveness (checked by the engine alive checker).")
+            }
           } catch {
-            case e: KyuubiSQLException =>
-              warn(s"Error closing session ${session.handle}", e)
+            case e: Throwable => warn(s"Error closing session ${session.handle}", e)
           }
-        }
+        case _ =>
       }
     }
-    engineConnectionAliveChecker.scheduleWithFixedDelay(
+    scheduleTolerableRunnableWithFixedDelay(
+      engineConnectionAliveChecker,
       checkTask,
       interval,
       interval,
