@@ -83,6 +83,8 @@ class JpsApplicationOperationSuite extends KyuubiFunSuite {
       val desc1 = jps.getApplicationInfoByTag(ApplicationManagerInfo(None), id)
       assert(desc1.id != null)
       assert(desc1.name != null)
+      assert(!desc1.name.contains("org.apache.spark.launcher.Main"))
+      assert(desc1.name.contains("org.apache.spark.deploy.SparkSubmit"))
       assert(desc1.state == ApplicationState.RUNNING)
       val response = jps.killApplicationByTag(ApplicationManagerInfo(None), id)
       assert(response._1, response._2)
