@@ -28,14 +28,14 @@ import scala.collection.JavaConverters._
 import org.scalatest.time.SpanSugar.convertIntToGrainOfTime
 
 import org.apache.kyuubi.{KyuubiFunSuite, RestFrontendTestHelper}
-import org.apache.kyuubi.client.api.v1.dto._
+import org.apache.kyuubi.client.api.v1.dto
+import org.apache.kyuubi.client.api.v1.dto.{SessionData, _}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_SESSION_CONNECTION_URL_KEY
 import org.apache.kyuubi.engine.ShareLevel
-import org.apache.kyuubi.events.KyuubiSessionEvent
 import org.apache.kyuubi.metrics.{MetricsConstants, MetricsSystem}
 import org.apache.kyuubi.operation.OperationHandle
-import org.apache.kyuubi.server.http.authentication.AuthenticationHandler.AUTHORIZATION_HEADER
+import org.apache.kyuubi.server.http.util.HttpAuthUtils.{basicAuthorizationHeader, AUTHORIZATION_HEADER}
 import org.apache.kyuubi.session.SessionType
 
 class SessionsResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
@@ -106,14 +106,9 @@ class SessionsResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
 
   test("get session event") {
     val sessionOpenRequest = new SessionOpenRequest(Map("testConfig" -> "testValue").asJava)
-
-    val user = "kyuubi".getBytes()
-
     val sessionOpenResp = webTarget.path("api/v1/sessions")
       .request(MediaType.APPLICATION_JSON_TYPE)
-      .header(
-        AUTHORIZATION_HEADER,
-        s"Basic ${new String(Base64.getEncoder().encode(user), StandardCharsets.UTF_8)}")
+      .header(AUTHORIZATION_HEADER, basicAuthorizationHeader("kyuubi"))
       .post(Entity.entity(sessionOpenRequest, MediaType.APPLICATION_JSON_TYPE))
 
     val sessionHandle = sessionOpenResp.readEntity(classOf[SessionHandle]).getIdentifier
@@ -121,10 +116,10 @@ class SessionsResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
     // get session event
     var response = webTarget.path(s"api/v1/sessions/$sessionHandle").request().get()
     assert(200 == sessionOpenResp.getStatus)
-    val sessions = response.readEntity(classOf[KyuubiSessionEvent])
-    assert(sessions.conf("testConfig").equals("testValue"))
-    assert(sessions.sessionType.equals(SessionType.INTERACTIVE.toString))
-    assert(sessions.user.equals("kyuubi"))
+    val sessions = response.readEntity(classOf[dto.KyuubiSessionEvent])
+    assert(sessions.getConf.get("testConfig").equals("testValue"))
+    assert(sessions.getSessionType.equals(SessionType.INTERACTIVE.toString))
+    assert(sessions.getUser.equals("kyuubi"))
 
     // close an opened session
     response = webTarget.path(s"api/v1/sessions/$sessionHandle").request().delete()
@@ -301,4 +296,89 @@ class SessionsResourceSuite extends KyuubiFunSuite with RestFrontendTestHelper {
       assert(sessionEvent.contains("The last 10 line(s) of log are:"))
     }
   }
+
+  test("fix kyuubi session leak caused by engine stop") {
+    // clean up all sessions
+    var response = webTarget.path("api/v1/sessions").request().get()
+    val sessionDataList = response.readEntity(new GenericType[List[SessionData]]() {})
+    sessionDataList.foreach(sessionData => {
+      response = webTarget.path(s"api/v1/sessions/${sessionData.getIdentifier}")
+        .request().delete()
+      assert(200 == response.getStatus)
+    })
+
+    // open a session
+    val requestObj = new SessionOpenRequest(Map(
+      KyuubiConf.ENGINE_ALIVE_PROBE_ENABLED.key -> "true",
+      KyuubiConf.ENGINE_ALIVE_PROBE_INTERVAL.key -> "5000",
+      KyuubiConf.ENGINE_ALIVE_TIMEOUT.key -> "3000").asJava)
+    response = webTarget.path("api/v1/sessions")
+      .request(MediaType.APPLICATION_JSON_TYPE)
+      .post(Entity.entity(requestObj, MediaType.APPLICATION_JSON_TYPE))
+    val sessionHandle = response.readEntity(classOf[SessionHandle]).getIdentifier
+    val pathPrefix = s"api/v1/sessions/$sessionHandle"
+
+    response = webTarget.path("api/v1/sessions/count").request().get()
+    val openedSessionCount = response.readEntity(classOf[SessionOpenCount])
+    assert(openedSessionCount.getOpenSessionCount == 1)
+
+    var statementReq = new StatementRequest(
+      "spark.sql(\"show tables\")",
+      true,
+      3000,
+      Collections.singletonMap(KyuubiConf.OPERATION_LANGUAGE.key, "SCALA"))
+    response = webTarget
+      .path(s"$pathPrefix/operations/statement").request(MediaType.APPLICATION_JSON_TYPE)
+      .post(Entity.entity(statementReq, MediaType.APPLICATION_JSON_TYPE))
+    assert(200 == response.getStatus)
+    var operationHandle = response.readEntity(classOf[OperationHandle])
+    assert(operationHandle !== null)
+    assert(openedSessionCount.getOpenSessionCount == 1)
+
+    statementReq = new StatementRequest(
+      "spark.close()",
+      true,
+      3000,
+      Collections.singletonMap(KyuubiConf.OPERATION_LANGUAGE.key, "SCALA"))
+    response = webTarget
+      .path(s"$pathPrefix/operations/statement").request(MediaType.APPLICATION_JSON_TYPE)
+      .post(Entity.entity(statementReq, MediaType.APPLICATION_JSON_TYPE))
+    assert(200 == response.getStatus)
+    operationHandle = response.readEntity(classOf[OperationHandle])
+    assert(operationHandle !== null)
+
+    // Because the engine has stopped (due to spark.close), the Spark session is closed.
+    // Therefore, the Kyuubi session count should be 0.
+    eventually(timeout(30.seconds), interval(1000.milliseconds)) {
+      var response = webTarget.path("api/v1/sessions/count").request().get()
+      val openedSessionCount = response.readEntity(classOf[SessionOpenCount])
+      assert(openedSessionCount.getOpenSessionCount == 0)
+
+      response = webTarget.path("api/v1/sessions").request().get()
+      val sessionDataList = response.readEntity(new GenericType[List[SessionData]]() {})
+      assert(sessionDataList.isEmpty)
+    }
+  }
+
+  test("list all type operations under session") {
+    val sessionOpenRequest = new SessionOpenRequest(Map("testConfig" -> "testValue").asJava)
+    val user = "kyuubi".getBytes()
+    val sessionOpenResp = webTarget.path("api/v1/sessions")
+      .request(MediaType.APPLICATION_JSON_TYPE)
+      .header(
+        AUTHORIZATION_HEADER,
+        s"Basic ${new String(Base64.getEncoder.encode(user), StandardCharsets.UTF_8)}")
+      .post(Entity.entity(sessionOpenRequest, MediaType.APPLICATION_JSON_TYPE))
+
+    val sessionHandle = sessionOpenResp.readEntity(classOf[SessionHandle]).getIdentifier
+
+    // get operations belongs to specified session
+    val response = webTarget
+      .path(s"api/v1/sessions/${sessionHandle.toString}/operations")
+      .request().get()
+    assert(200 == response.getStatus)
+    val operations = response.readEntity(new GenericType[Seq[OperationData]]() {})
+    assert(operations.size == 1)
+    assert(sessionHandle.toString.equals(operations.head.getSessionId))
+  }
 }
